@@ -289,6 +289,13 @@ seahorse_keyset_new_full (SeahorseKeyPredicate *pred)
     return g_object_new (SEAHORSE_TYPE_KEYSET, "predicate", pred, NULL);
 }
 
+gboolean
+seahorse_keyset_has_key (SeahorseKeyset *skset, SeahorseKey *skey)
+{
+    return g_hash_table_lookup (skset->pv->keys, skey) ? 
+                TRUE : FALSE;
+}
+
 GList*             
 seahorse_keyset_get_keys (SeahorseKeyset *skset)
 {
@@ -377,7 +384,7 @@ pgp_signers_match (SeahorseKey *key, gpointer data)
     SeahorseKey *defkey = seahorse_context_get_default_key (SCTX_APP ());
     
     /* Default key overrides all, and becomes the only signer available*/
-    if (defkey && strcmp (seahorse_key_get_keyid (key), seahorse_key_get_keyid (defkey)) != 0)
+    if (defkey && seahorse_key_get_keyid (key) != seahorse_key_get_keyid (defkey))
         return FALSE;
     
     return TRUE;
@@ -399,7 +406,8 @@ seahorse_keyset_pgp_signers_new ()
     skset = seahorse_keyset_new_full (pred);
     g_object_set_data_full (G_OBJECT (skset), "pgp-signers-predicate", pred, g_free);
     
-    seahorse_gconf_notify_lazy (DEFAULT_KEY, (GConfClientNotifyFunc)pgp_signers_gconf_notify, 
+    seahorse_gconf_notify_lazy (SEAHORSE_DEFAULT_KEY, 
+                                (GConfClientNotifyFunc)pgp_signers_gconf_notify, 
                                 skset, skset);
     return skset;
 }
