@@ -111,13 +111,20 @@ operation_done (SeahorseOperation *operation, GtkWidget *appbar)
     operation_progress (operation, "", 0.0, appbar);    
 }
 
+static void
+disconnect_progress (GtkWidget *widget, SeahorseOperation *op)
+{
+    g_signal_handlers_disconnect_by_func (op, operation_progress, widget);
+    g_signal_handlers_disconnect_by_func (op, operation_done, widget);
+}
+
 void 
 seahorse_progress_appbar_set_operation (GtkWidget* appbar, SeahorseOperation *operation)
 {
     g_return_if_fail (GNOME_IS_APPBAR (appbar));
     g_return_if_fail (SEAHORSE_IS_OPERATION (operation));
     
-    if (seahorse_operation_is_done (operation)) {
+    if (!seahorse_operation_is_running (operation)) {
         operation_done (operation, appbar);
         return;
     }
@@ -126,8 +133,9 @@ seahorse_progress_appbar_set_operation (GtkWidget* appbar, SeahorseOperation *op
     g_signal_connect (operation, "progress", G_CALLBACK (operation_progress), appbar);
     g_object_set_data_full (G_OBJECT (appbar), "operations", operation, 
                                 (GDestroyNotify)g_object_unref);
+    g_signal_connect (appbar, "destroy", G_CALLBACK (disconnect_progress), operation);
 
-    operation_progress (operation, seahorse_operation_get_details (operation),
+    operation_progress (operation, seahorse_operation_get_message (operation),
                         seahorse_operation_get_progress (operation), appbar);
 }
 
@@ -146,7 +154,7 @@ progress_operation_update (SeahorseOperation *operation, const gchar *message,
     w = glade_xml_get_widget (swidget->xml, "operation-details");
     g_return_if_fail (w != NULL);
     
-    t = seahorse_operation_get_details (operation);
+    t = seahorse_operation_get_message (operation);
     gtk_label_set_text (GTK_LABEL (w), t ? t : "");
     
     progress = GTK_PROGRESS_BAR (glade_xml_get_widget (swidget->xml, "operation-bar"));
@@ -163,7 +171,7 @@ progress_operation_update (SeahorseOperation *operation, const gchar *message,
 static void
 progress_operation_cancel (GtkButton *button, SeahorseOperation *operation)
 {
-    if (!seahorse_operation_is_done (operation))
+    if (seahorse_operation_is_running (operation))
         seahorse_operation_cancel (operation);
 }
 
@@ -204,13 +212,13 @@ progress_show (SeahorseOperation *operation)
     const gchar *title;
     gchar *t;
 
-    if (seahorse_operation_is_done (operation)) {
+    if (!seahorse_operation_is_running (operation)) {
         /* Matches the ref in seahorse_progress_show */
         g_object_unref (operation);
         return FALSE;
     }
         
-    swidget = seahorse_widget_new ("progress");
+    swidget = seahorse_widget_new_allow_multiple ("progress");
     g_return_val_if_fail (swidget != NULL, FALSE);
 
     /* Release our reference on the operation when this window is destroyed */    
@@ -218,7 +226,7 @@ progress_show (SeahorseOperation *operation)
                             (GDestroyNotify)g_object_unref);    
         
     /* Setup the title */
-    title = (const gchar*)g_object_get_data (G_OBJECT (operation), "title");
+    title = (const gchar*)g_object_get_data (G_OBJECT (operation), "progress-title");
     if (title) {
             
         /* The window title */
@@ -242,9 +250,9 @@ progress_show (SeahorseOperation *operation)
 
     /* Cancel events */
     glade_xml_signal_connect_data (swidget->xml, "cancel_clicked",
-	                               G_CALLBACK (progress_operation_cancel), operation);
-    glade_xml_signal_connect_data (swidget->xml, "delete_event",
-	                               G_CALLBACK (progress_delete_event), operation);
+                                   G_CALLBACK (progress_operation_cancel), operation);
+    g_signal_connect (seahorse_widget_get_top (swidget), "delete_event",
+                      G_CALLBACK (progress_delete_event), operation);
     
     /* Done and cleanup */
     w = glade_xml_get_widget (swidget->xml, swidget->name);
@@ -260,7 +268,7 @@ seahorse_progress_show (SeahorseOperation *operation, const gchar *title,
 {    
     /* Unref in the timeout callback */
     g_object_ref (operation);
-    g_object_set_data_full (G_OBJECT (operation), "title", 
+    g_object_set_data_full (G_OBJECT (operation), "progress-title", 
                 title ? g_strdup (title) : NULL, (GDestroyNotify)g_free);
     
     /* Show the progress, after one second */
