@@ -40,9 +40,8 @@ const AvahiPoll* seahorse_util_dns_sd_get_poll ();
 #endif
 
 typedef enum {
-	SEAHORSE_CRYPT_SUFFIX,
-	SEAHORSE_SIG_SUFFIX,
-	SEAHORSE_ASC_SUFFIX
+    SEAHORSE_CRYPT_SUFFIX,
+    SEAHORSE_SIG_SUFFIX,
 } SeahorseSuffix;
 
 #define SEAHORSE_EXT_ASC ".asc"
@@ -72,12 +71,8 @@ void        seahorse_util_handle_gpgme          (gpgme_error_t      err,
 
 void        seahorse_util_handle_error          (GError*            err,
                                                  const char*        desc, ...);
-                                                 
-gpgme_error_t	seahorse_util_write_data_to_file (const gchar       *path,
-							                     gpgme_data_t       data,
-                                                 gboolean           release);
 
-gchar*		seahorse_util_write_data_to_text	(gpgme_data_t       data,
+gchar*      seahorse_util_write_data_to_text    (gpgme_data_t       data,
                                                  gboolean           release);
 
 guint       seahorse_util_read_data_block       (GString            *buf, 
@@ -158,6 +153,12 @@ GSList*     seahorse_util_string_slist_copy   (GSList *slist);
 
 gboolean    seahorse_util_string_slist_equal  (GSList *sl1, GSList *sl2);
 
+void        seahorse_util_determine_popup_menu_position  (GtkMenu *menu,
+                                                           int *x,
+                                                           int *y,
+                                                           gboolean *push_in,
+                                                           gpointer gdata);
+
 #define     seahorse_util_wait_until(expr)          \
     while (!(expr)) {                               \
         g_thread_yield ();                          \
