@@ -26,16 +26,17 @@
 
 #include "seahorse-widget.h"
 #include "config.h"
+#include "seahorse-gtkstock.h"
 
 #define STATUS "status"
 
 enum {
 	PROP_0,
-	PROP_NAME,
-	PROP_CTX
+	PROP_NAME
 };
 
 static void     class_init          (SeahorseWidgetClass    *klass);
+static void     object_init         (SeahorseWidget         *swidget);
 
 static void     object_finalize     (GObject                *gobject);
 
@@ -63,7 +64,7 @@ static gboolean widget_delete_event  (GtkWidget             *widget,
 static void     context_destroyed    (GtkObject             *object,
                                       SeahorseWidget        *swidget);
 
-static GObjectClass *parent_class = NULL;
+static GtkObjectClass *parent_class = NULL;
 
 /* Hash of widgets with name as key */
 static GHashTable *widgets = NULL;
@@ -77,10 +78,10 @@ seahorse_widget_get_type (void)
 		static const GTypeInfo widget_info = {
 			sizeof (SeahorseWidgetClass), NULL, NULL,
 			(GClassInitFunc) class_init,
-			NULL, NULL, sizeof (SeahorseWidget), 0, NULL
+			NULL, NULL, sizeof (SeahorseWidget), 0, (GInstanceInitFunc) object_init
 		};
 		
-        widget_type = g_type_register_static (G_TYPE_OBJECT, "SeahorseWidget", 
+        widget_type = g_type_register_static (GTK_TYPE_OBJECT, "SeahorseWidget", 
                                               &widget_info, 0);
 	}
 	
@@ -102,9 +103,20 @@ class_init (SeahorseWidgetClass *klass)
     g_object_class_install_property (gobject_class, PROP_NAME,
         g_param_spec_string ("name", "Widget name", "Name of glade file and main widget",
                              NULL, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-    g_object_class_install_property (gobject_class, PROP_CTX,
-        g_param_spec_object ("ctx", "Seahorse Context", "Current Seahorse Context to use",
-                             SEAHORSE_TYPE_CONTEXT, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+}
+
+/* Destroy widget when context is destroyed */
+static void
+context_destroyed (GtkObject *object, SeahorseWidget *swidget)
+{
+	seahorse_widget_destroy (swidget);
+}
+
+static void
+object_init (SeahorseWidget *swidget)
+{
+    g_signal_connect_after (SCTX_APP(), "destroy", 
+                G_CALLBACK (context_destroyed), swidget);
 }
 
 /* Disconnects callbacks, destroys main window widget,
@@ -124,16 +136,14 @@ object_finalize (GObject *gobject)
     		widgets = NULL;
     	}
     }
-    	
-	g_signal_handlers_disconnect_by_func (swidget->sctx, context_destroyed, swidget);
+
+	g_signal_handlers_disconnect_by_func (SCTX_APP (), context_destroyed, swidget);
     if (glade_xml_get_widget (swidget->xml, swidget->name))
-    	gtk_widget_destroy (glade_xml_get_widget (swidget->xml, swidget->name));
+	    gtk_widget_destroy (glade_xml_get_widget (swidget->xml, swidget->name));
 	
-	g_free (swidget->xml);
+	g_object_unref (swidget->xml);
 	swidget->xml = NULL;
 	
-	g_object_unref (swidget->sctx);
-	
 	g_free (swidget->name);
 	
 	G_OBJECT_CLASS (parent_class)->finalize (gobject);
@@ -144,6 +154,7 @@ object_set_property (GObject *object, guint prop_id, const GValue *value, GParam
 {
 	SeahorseWidget *swidget;
     GtkWidget *w;
+    GdkPixbuf *pixbuf = NULL;
 	char *path;
 	
 	swidget = SEAHORSE_WIDGET (object);
@@ -169,17 +180,15 @@ object_set_property (GObject *object, guint prop_id, const GValue *value, GParam
             w = glade_xml_get_widget (swidget->xml, swidget->name);
             glade_xml_set_toplevel (swidget->xml, GTK_WINDOW (w));
             glade_xml_ensure_accel (swidget->xml);
-        
-            gtk_window_set_icon (GTK_WINDOW (w), gdk_pixbuf_new_from_file (PIXMAPSDIR "seahorse.png", NULL));
+            
+            pixbuf = gtk_widget_render_icon (w, 
+                                     SEAHORSE_STOCK_SEAHORSE, 
+                                     (GtkIconSize)-1, 
+                                     NULL); 
+            gtk_window_set_icon (GTK_WINDOW (w), gdk_pixbuf_copy(pixbuf));
+            g_object_unref(pixbuf);
 			break;
             
-		case PROP_CTX:
-            g_return_if_fail (swidget->sctx == NULL);
-			swidget->sctx = g_value_get_object (value);
-			g_object_ref (G_OBJECT (swidget->sctx));
-			g_signal_connect_after (swidget->sctx, "destroy",
-				G_CALLBACK (context_destroyed), swidget);
-			break;
 		default:
 			break;
 	}
@@ -195,9 +204,6 @@ object_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *
 		case PROP_NAME:
 			g_value_set_string (value, swidget->name);
 			break;
-		case PROP_CTX:
-			g_value_set_object (value, swidget->sctx);
-			break;
 		default:
 			break;
 	}
@@ -224,63 +230,66 @@ widget_delete_event (GtkWidget *widget, GdkEvent *event, SeahorseWidget *swidget
     return FALSE; /* propogate event */
 }
 
-/* Destroy widget when context is destroyed */
-static void
-context_destroyed (GtkObject *object, SeahorseWidget *swidget)
-{
-	seahorse_widget_destroy (swidget);
-}
-
 /**
  * seahorse_widget_new:
  * @name: Name of widget, filename part of glade file, and name of main window
- * @sctx: #SeahorseContext
  *
  * Creates a new #SeahorseWidget.
  *
  * Returns: The new #SeahorseWidget, or NULL if the widget already exists
  **/
 SeahorseWidget*
-seahorse_widget_new (gchar *name, SeahorseContext *sctx)
+seahorse_widget_new (const gchar *name)
 {
-	SeahorseWidget *swidget;
-	
-	g_return_val_if_fail (SEAHORSE_IS_CONTEXT (sctx), NULL);
-	
-	/* Check if have widget hash */
-	if (widgets != NULL) {
-		swidget = g_hash_table_lookup (widgets, name);
-		
-		/* If widget already exists, present */
-		if (swidget != NULL) {
-			gtk_window_present (GTK_WINDOW (glade_xml_get_widget (swidget->xml, swidget->name)));
-			return NULL;
-		}
-	}
-	/* Else create new widget hash */
-	else
-		widgets = g_hash_table_new ((GHashFunc)g_str_hash, (GCompareFunc)g_str_equal);
-	
-	/* If widget doesn't already exist, create & insert into hash */
-	swidget = g_object_new (SEAHORSE_TYPE_WIDGET, "name", name, "ctx", sctx, NULL);
-	g_hash_table_insert (widgets, g_strdup (name), swidget);
-	
-	return swidget;
+    /* Check if have widget hash */
+    SeahorseWidget *swidget = seahorse_widget_find (name);
+    
+    /* If widget already exists, present */
+    if (swidget != NULL) {
+        gtk_window_present (GTK_WINDOW (glade_xml_get_widget (swidget->xml, swidget->name)));
+        return NULL;
+    }
+
+    /* If widget doesn't already exist, create & insert into hash */
+    swidget = g_object_new (SEAHORSE_TYPE_WIDGET, "name", name, NULL);
+    if(!widgets)
+        widgets = g_hash_table_new ((GHashFunc)g_str_hash, (GCompareFunc)g_str_equal);
+    g_hash_table_insert (widgets, g_strdup (name), swidget);
+
+    /* We don't care about this floating business */
+    g_object_ref (GTK_OBJECT (swidget));
+    gtk_object_sink (GTK_OBJECT (swidget));
+
+    return swidget;
 }
 
 /**
  * seahorse_widget_new:
  * @name: Name of widget, filename part of glade file, and name of main window
- * @sctx: #SeahorseContext
  *
  * Creates a new #SeahorseWidget without checking if it already exists.
  *
  * Returns: The new #SeahorseWidget
  **/
 SeahorseWidget*
-seahorse_widget_new_allow_multiple (gchar *name, SeahorseContext *sctx)
+seahorse_widget_new_allow_multiple (const gchar *name)
 {
-	return g_object_new (SEAHORSE_TYPE_WIDGET, "name", name, "ctx", sctx, NULL);
+    SeahorseWidget *swidget = g_object_new (SEAHORSE_TYPE_WIDGET, "name", name,  NULL);
+    
+    /* We don't care about this floating business */
+    g_object_ref (GTK_OBJECT (swidget));
+    gtk_object_sink (GTK_OBJECT (swidget));
+
+    return swidget;
+}
+
+SeahorseWidget*
+seahorse_widget_find (const gchar *name)
+{
+    /* Check if have widget hash */
+    if (widgets != NULL)
+        return SEAHORSE_WIDGET (g_hash_table_lookup (widgets, name));
+    return NULL;
 }
 
 /**
@@ -295,7 +304,7 @@ seahorse_widget_show_help (SeahorseWidget *swidget)
     GError *err = NULL;
 
     if (g_str_equal (swidget->name, "key-manager"))
-        gnome_help_display (PACKAGE, NULL, &err);
+        gnome_help_display (PACKAGE, "introduction", &err);
     else
         gnome_help_display (PACKAGE, swidget->name, &err);
 
@@ -329,6 +338,14 @@ seahorse_widget_get_top     (SeahorseWidget     *swidget)
     return widget;
 }
 
+GtkWidget*
+seahorse_widget_get_widget (SeahorseWidget *swidget, const char *identifier)
+{
+    GtkWidget *widget = glade_xml_get_widget (swidget->xml, identifier);
+    g_return_val_if_fail (widget != NULL, NULL);
+    return widget;
+}
+
 /**
  * seahorse_widget_show:
  * @swidget: #SeahorseWidget to show
@@ -347,6 +364,19 @@ seahorse_widget_show (SeahorseWidget *swidget)
     gtk_widget_show (widget);
 }
  
+void             
+seahorse_widget_set_visible (SeahorseWidget *swidget, const char *identifier,
+                             gboolean visible)
+{
+    GtkWidget *widget = glade_xml_get_widget (swidget->xml, identifier);
+    g_return_if_fail (widget != NULL);
+    
+    if (visible)
+        gtk_widget_show (widget);
+    else
+        gtk_widget_hide (widget);
+}
+
 /**
  * seahorse_widget_destroy:
  * @swidget: #SeahorseWidget to destroy
